class InfoBox
{
  World world;
  int boxWidth, boxMargin;
  int titleHeight;
  int barHeight, barSpacing;

  InfoBox(World w)
  {
    world = w;
    boxWidth = 100;
    boxMargin = 2;
    titleHeight = 62;
    barHeight = 12;
    barSpacing = 5;
  }

  // elements infos
  void display(Element e)
  {
    display(e, int(e.pos.x), int(e.pos.y), e.rad);
  }
  void display(Element e, int targetX, int targetY, float biais)
  {
    // data
    PVector test = new PVector(targetX, targetY);
    test.normalize();
    test.mult(e.rad);
    int x = int(targetX + test.x);
    int y = int(targetY + test.y);
    Genome g = e.genes;
    int bars = g.size() + 1; // genome bars + energy bar
    int boxHeight = titleHeight + bars * (barHeight + barSpacing) - barSpacing + boxMargin;
 
    // box
    fill(255, 200);
    stroke(0);
    strokeWeight(1);
    if(x + boxWidth > width) x = int(targetX - boxWidth - biais);
    if(y + boxHeight > height) y = int(targetY - boxHeight - biais);
    rect(x, y, boxWidth, boxHeight);
    
    stroke(e.col);
    noFill();
    ellipse(e.pos.x, e.pos.y, e.rad * 2 + 20, e.rad * 2 + 20);
    
    // title
    String title;
    if(e instanceof Boid) title = "Boid, ";
    else if(e instanceof Food) title = "Food, ";
    else title = "";
    if(world.elements.indexOf(e) == -1) title += "#" + (world.deads.indexOf(e)+1);
    else title += "Element #" + world.elements.indexOf(e);

    // attrs
    fill(0);
    text(title + "\nAge :  "+ round(e.age) + "\nMass :  "+ round(e.mass) + "\nChilds : " + e.childs.size(),
    x + boxMargin, y + boxMargin, boxWidth - boxMargin, titleHeight - boxMargin);

    // energy bar
    bar(x + boxMargin, y + titleHeight, boxWidth - 2 * boxMargin, barHeight, 0, g.get("energyStart", 2), e.energy, "energy");
    // genome-based bars
    for(int i=0; i < e.genes.size(); i++)
    {
      Gene gene = (Gene)g.get(i);
      bar(
      x + boxMargin,
      y + (1+i) * (barHeight + barSpacing) + titleHeight,
      boxWidth - 2 * boxMargin,
      barHeight,
      gene.get(1),
      gene.get(2),
      gene.get(3),
      gene.name()
        );
    }
  }

  // world infos in mouse position
  void display()
  {
    display(mouseX, mouseY);
  }
  // world infos in specified position
  void display(int targetX, int targetY)
  {
    fill(255);
    stroke(0);
    strokeWeight(1);
    int w = 120;
    int h = 95; 
    float x = targetX + 10;
    float y = targetY + 20;
    if(x + w > width) x =  targetX - w ;
    if(y + h > height) y = targetY - h ;
    rect(x, y, w, h);

    int margin = 3;
    int r = 2;
    fill(0);
    boolean play;
    text("World"+
      "\nSize : "+world.w+"x"+world.h+
      "\nAlive elements : "+world.elements.size()+
      "\nDead elements : "+world.deads.size()+
      "\nFriction : "+world.friction+
      "\nLight : "+world.light,
    x + margin, y + margin, w - margin, h - margin);
  }

  // display infos about dead elements in all world surface
  void deads()
  {
    Element e, e2;
    int number = 10;
    int margin = 17;
    int boxHeight;
    int col = 0, row = 0, bars = 0;

    // background
    fill(255);
    strokeWeight(margin);
    rect(world.x, world.y, world.w, world.h);
    
    for(int i=0; i < 5; i++)
    {
      int posX = world.w - ((i+2) * boxWidth);

      e = (Element)world.deads.get(i);
      display(e, posX , margin + 60, 0);
      e.pos = new PVector(posX + boxWidth / 2, margin + 30);
      e.display();
      
      e2 = (Element)world.deads.get(i+5);
      display(e2, posX , world.y + world.h - 270 - margin, 0);
      e2.pos = new PVector(posX + boxWidth / 2, world.y + world.h - 300 - margin);
      e2.display();
      
    }
  }

  void bar(int x, int y, int w, int h, float min, float max, float value, String title)
  {
    // border
    fill(255);
    stroke(0);
    strokeWeight(1);
    rect(x, y, w, h);

    // bar
    float minVal = value - min;
    float maxVal = abs(max-min);
    int barWidth = int(minVal * w / maxVal - 1);

    if(barWidth >= 1) // 1 pixel min
    {
      fill(255 - (minVal * 255 / maxVal), minVal * 255 / maxVal, 0);
      noStroke();
      // multi-bar
      if(minVal > maxVal)
      {
        float d = minVal / maxVal;
        float m = minVal % maxVal;
        rect(x+1, y+1, w - 1, h - 1);
        fill(minVal * 255 / maxVal, minVal * 255 / maxVal, d * 20);
        rect(x+1, y+1, m * w / maxVal, h - 1);  
      }  
      else rect(x+1, y+1, barWidth, h - 1);
    }

    // title
    fill(0);
    text(title, x+2, y, w-4, h);
  }
}





