/**
 * Zone class for Processing, 
 * Nicolas Liautaud 2009
 * <br/>
 * Used by BoidsFlock class
 */
class World
{
  int x, y, w, h;
  ArrayList elements, deads;
  Element selected;
  float friction;
  float light;
  boolean play;
  InfoBox infos;

  World()
  {
    x = 0;
    y = 0;
    w = width;
    h = height;
    elements = new ArrayList();
    deads = new ArrayList();
    friction = 0.5;
    light = 1;
    play = true;
    selected = null;
    infos = new InfoBox(this);
  }

  // Add an element to world.
  void add(Element e)
  {
    elements.add(e); 
  }

  // Add specified number of elements to world.
  void add(Element e, int n)
  {
    for(int i=0; i < n; i++)
      elements.add(e); 
  }

  // Delete element from world.
  void die(Element e)
  {
    die(elements.indexOf(e));
  }
  void die(int i)
  {
    if(i!=-1)
    {
      Element e = (Element)elements.remove(i);
      deads.add(e);
      Collections.sort(deads);
    }
  }
  // Kill all.
  void kaboom()
  {
    int s = elements.size();
    for(int i=0; i < s; i++) die(0);
  }

  // Return the number of elements in world.
  int size()
  {
    return elements.size();
  }

  // Return the specified element.
  Element get(int i)
  {
    return (Element)elements.get(i);
  }

  // Return a random pos vector in world.
  PVector randomPos()
  {
    return new PVector(random(x, x + w), random(y, y + h)); 
  }

  // Return the element under mouse, or null.
  Element isHover()
  {
    return isHover(mouseX, mouseY);
  }
  Element isHover(int x, int y)
  {
    Element hover = null;
    for(int i = 0; i < elements.size(); i++)
    {
      Element e = (Element)get(i);
      if(dist(x, y, e.pos.x, e.pos.y) < e.rad)
      {
        hover = e;
        break;
      }
    }
    return hover;
  }

  // Update all elements in world.
  void update()
  {
    if(play)
    {
      for(int i = 0; i < elements.size(); i++)
        ((Element)get(i)).update();   
    }
  }

  // Display a border around world.
  void displayBorder(int a)
  {
    noFill(); 
    stroke(a);
    rect(x, y, w - 1, h - 1); 
  }

  // Display all elements in world.
  void display()
  {
    for(int i = 0; i < elements.size(); i++)
      ((Element)get(i)).display();

    if(selected != null){
      infos.display(selected);
    }
    else {
      Element e = isHover();
      if(e != null) infos.display(e);
      else if(mousePressed && mouseButton == RIGHT)
        infos.display();
    }
    
    // pause logo
    if(!play)
    {
      fill(50,60);
      stroke(100,60);
      strokeWeight(10);
      rect((width-200)/2, (height-200)/2, 200, 200);
      text("Pause", width/2-15, height/2-5,30,12);
      strokeWeight(40);
      stroke(100,60);
      line((width-200)/2 + 60, (height-200)/2 + 50, (width-200)/2 + 60, (height-200)/2 + 150 );
      line((width-200)/2 + 140, (height-200)/2 + 50, (width-200)/2 + 140, (height-200)/2 + 150 );
    }
  }
}











