import processing.core.*; 
import processing.xml.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class boids6 extends PApplet {

/**
 * Evolution, 
 * Nicolas Liautaud 2009
 */

World world;
Element selected;
int foodNbr, boidNbr;

boolean displayLinks = true;

public void setup() 
{ 
  // canvas
  size(700, 700);
  smooth();

  foodNbr = 100;
  boidNbr = 10;

  // initialize fps counter
  fps = new FpsCounter(width - 50, 10);

  // initialize simulation world
  world = new World();

  // create foods
  for(int i=0; i < foodNbr; i++)
    world.add(new Food(world));
  // create boids
  for(int i=0; i < boidNbr; i++)
    world.add(new Boid(world));
}

public void draw() 
{
  background(255);

  // display and update world
  world.display();
  world.update();

  // end by replicators invasion (kill all at low framerate)
  if(world.deads.size() > 0 && frameRate < 20)
    world.kaboom();
  // end by no survivors
  if(world.size() < 1)
  {
    world.play = false;  // pause
    world.infos.deads(); // display winners
  }

  fps.display();
} 

// restart, with the 10 bests of the precedent world if there was
public void restart()
{
  // if there is deads
  if(world.deads.size() > 0)
  {
    List winners;
    int deadsNbr = world.deads.size();
    // copy the 10 first if there enough, else copy what is it available
    if(deadsNbr >= 9) winners = world.deads.subList(0, 9);
    else winners = world.deads.subList(0, deadsNbr);
    // re-initialize world
    world = new World();
    // add winners with start energy to new world
    for(int i=0; i < winners.size(); i++)
    {
      Element e = (Element)winners.get(i);
      // e.genes.set("energy", e.genes.get("energyStart"));
      world.add(e);
    }
  }
  else world = new World();

  // create foods
  for(int i=0; i < foodNbr; i++)
    world.add(new Food(world));
  // create boids
  for(int i=0; i < boidNbr; i++)
    world.add(new Boid(world));
}

public void mousePressed()
{  
  selected = world.isHover(mouseX, mouseY);
  if(selected != null) // an element is cliqued
  {
    // delete it with middle clic
    if(mouseButton == CENTER)
      world.die(selected);
    // select it with left clic
    else if(mouseButton == LEFT)
    {
      // if already selected, unselect-it
      if(world.selected == selected)
        world.selected = null;
      // else select-it
      else world.selected = selected;
    }
  }
  else // nothing is selected
  {
    // add food with left clic
    if(mouseButton == LEFT)
      world.add(new Food(world, new PVector(mouseX, mouseY)));
  }
}
public void mouseDragged()
{  
  if(selected != null)
  {
    selected.pos.x = mouseX;
    selected.pos.y = mouseY;
    selected.vel.x = mouseX - pmouseX;
    selected.vel.y = mouseY - pmouseY;
    world.selected = null;
  }
}

public void keyPressed()
{
  // reset on SPACE key
  if(key == ' ')
  {
    restart();
    loop();
  }

  // pause on P key
  if(key == 'p' || key == 'P')
  {
    if(world.play) world.play = false;
    else world.play = true;
  }

  // kill all on K key
  if(key == 'k' || key == 'K')
    world.kaboom();

  // display parent/child links on L key
  if(key == 'l' || key == 'L')
  {
    if(displayLinks) displayLinks = false;
    else displayLinks = true;
  }
}



/**
 * Boid class for Processing, 
 * Nicolas Liautaud 2009
 * <br/>
 * Used by BoidsFlock class
 */

class Boid extends Element
{
  Boid(World world)
  {
    super(world);
    genes.add("cohesion", 0, 10);
    genes.add("separation", 0, 10);
    genes.add("imitation", 0, 10);
    genes.add("perception", 0, world.w);
  }
  Boid(Boid parent)
  {
    super(parent);
  }

  public void update()
  { 
    float energyStart = genes.get("energyStart");
    float feedEffect = genes.get("feedEffect");
    
    // reproduction
    if(energy > 1.5f * energyStart && genes.get("reproductionAbility") > random(2))
    {
      new Boid(this);
      energy += genes.get("reproductionEffect");
    }
    
    // feed
    for(int i=0; i < world.size(); i++)
    {
      Element e = (Element)world.get(i);
      if(e instanceof Food)
      {
        // medium attraction when perceived
        effector(PApplet.parseInt(e.pos.x), PApplet.parseInt(e.pos.y), e.rad + genes.get("perception"), genes.get("feedAbility"));
        // big repulsion when touched
        effector(PApplet.parseInt(e.pos.x), PApplet.parseInt(e.pos.y), e.rad + rad, -20);
        // eat it at contact
        if(e.distance(this) <= rad + e.rad)
        {
          energy += feedEffect;
          e.energy -= feedEffect;
        }
      }
    }

    // do all other things from super class
    super.update();

    // boids rules
    keepDistance(childs, genes.get("separation") * 10, PApplet.parseInt(genes.get("separation")));
    cohesion(childs, genes.get("cohesion") - 5);
    matchVelocity(childs, genes.get("imitation") / 10);
  }

  /**
   * <p>Boid imitation : try to move in the same way than other boids.</p>
   * <p>Compute the average velocity of the percepted
   * flock, and add a part of that to velocity.</p>
   */
  public void matchVelocity(ArrayList boids, float intensity)
  {
    PVector v = new PVector();
    int nbr = 0;
    for(int i=0; i < boids.size(); i++)
    { 
      Element b = (Element) boids.get(i);
      if(b != this && PVector.dist(pos, b.pos) < genes.get("perception"))
      {
        // add velocity of each other percepted boid
        v.add(b.vel);
        nbr++;
      }
    }
    if(nbr > 0)
    {
      v.div(nbr); // do average
      v.sub(vel);
      v.mult(intensity); // set intensity
      vel.add(v); // and add to velocity
    }
  }

  public void limitVelocity(float lim){
    if(vel.mag() > lim)
      vel = PVector.mult(PVector.div(vel, vel.mag()), lim);
  }

  /**
   * Affect boid velocity with an linear point force.
   *
   * @param  x x-coordinate of the effector
   * @param  y y-coordinate of the effector
   * @param  a aura of the effector (effect distance)
   * @param  i intensity of the effector. > 0 for attraction, < 0 for repulsion
   */
  public void effector(int x, int y, float a, float e)
  {
    float d = dist(x, y, pos.x, pos.y);
    if(d < a)
    {
      int f = 100;
      PVector v = new PVector();
      if(pos.x >= x) v.x += (a-d) * e / f;
      else v.x -= (a-d) * e / f;
      if(pos.y >= y) v.y += (a-d) * e / f;
      else v.y -= (a-d) * e / f;
      vel.sub(v);
    }
  }

  // add the vector display to the super class display.
  public void display()
  {
    strokeWeight(2);
    super.display();
    PVector v = vel.get();
    v.normalize();
    v.mult(rad/2);
    strokeWeight(rad);
    stroke(col);
    point(pos.x + v.x, pos.y + v.y);
  }
}


class Element implements Comparable
{
  World world;
  PVector pos, vel;
  float mass, energy, rad, age;
  ArrayList childs;
  int col;
  Genome genes;

  Element(World world)
  {
    this(world, world.randomPos()); 
  }
  Element(World w, PVector p)
  {
    world = w;
    pos = p;
    vel = new PVector(random(1), random(1));
    rad = random(4, 15);
    mass = random(rad/2, rad*2);
    col = color(random(0,255),0,random(0,255));

    genes = new Genome();
    float eS = genes.add("energyStart", 100, 1000);
    genes.add("health", -1, 0);
    genes.add("feedEffect", 0, 1);
    genes.add("feedAbility", 0, 1);
    genes.add("reproductionEffect", -eS, -eS/4);
    genes.add("reproductionAbility", 0, 1);

    energy = eS;
    age = 0;
    childs = new ArrayList();
  }
  /**
   * Element constructor derivated from parent element.
   */
  Element(Element parent)
  {
    world = parent.world;
    world.add(this);
    pos = PVector.add(parent.pos, new PVector(random(-1,1), random(-1,1)));
    vel = PVector.add(parent.vel, new PVector(random(-1,1), random(-1,1)));
    rad = parent.rad;
    mass = rad;
    col = parent.col;
    genes = parent.genes;

    energy = genes.get("energyStart");
    age = 0;
    childs = new ArrayList();
    parent.childs.add(this);
  }

  public void update()
  {
    // life
    age += 1 / frameRate;
    energy += (age * genes.get("health")) / frameRate;

    // death
    if(energy < 1)
    {
      world.die(this);
    }

    // physics
    keepInWorld(world, 20);
    keepDistance(world.elements, 10, 1);
    keepDistance(childs, 20, 3);
    vel.mult(world.friction);
    vel.div(mass);
    pos.add(vel);
  }
  
  /**
   * <p>Element cohesion : try to approach other elements.</p>
   * <p>Compute the average direction to the barycenter of the
   * percepted flock, and add a part of that to velocity.</p>
   */
  public void cohesion(ArrayList a, float intensity)
  {
    PVector v = new PVector();
    int nbr = 0;
    for(int i=0; i < a.size(); i++)
    {
      Element e = (Element)a.get(i);
      if(e != this && (e instanceof Food || (e instanceof Boid && this.distance(e) < genes.get("perception"))))
      {
        // add position of each other percepted boid
        v.add(e.pos);
        nbr++;
      }
    }
    if(nbr > 0)
    {
      v.div(nbr); // do average
      v.sub(pos);
      v.mult(intensity); // set intensity
      v.div(mass);
      vel.add(v); // and add to velocity
    }
  }

  /**
   * <p>Element avoidance : try to keep a minimum distance between others.</p>
   * <p>Compute the average opposite way to all the touched
   * elements, and add a part of that to velocity.</p>
   */
  public void keepDistance(ArrayList a, float intensity, int m)
  {
    PVector u, v = new PVector();
    for(int i=0; i < a.size(); i++)
    {
      Element e = (Element)a.get(i);
      float d = distance(e);
      if(d != 0 && d < rad * m + e.rad)
      {
        u = PVector.sub(pos, e.pos);
        u.normalize();
        u.div(d);
        u.mult(e.mass);
        v.add(u);
      }
    }
    v.mult(intensity); // set intensity
    vel.add(v); // and add to velocity
  }

  /**
   * Keep the element in world by bouncing on sides.
   * i ~ 1.0 : hard
   * i ~ 0.1 : smooth
   */
  public void keepInWorld(World world, float i)
  {
    int maxX = world.x + world.w;
    int maxY = world.y + world.h;
    PVector v = new PVector();
    // When a boid approach a side, an opposite vector is added to velocity
    if(pos.x <= world.x + rad)   v.x = (world.x + rad - pos.x) * i;
    else if(pos.x >= maxX - rad) v.x = (maxX - rad - pos.x) * i;
    if(pos.y <= world.y + rad)   v.y = (world.y + rad - pos.y) * i;
    else if(pos.y >= maxY - rad) v.y = (maxY - rad - pos.y) * i;
    vel.add(v);

  }

  public void display()
  {
    fill(col, 50);
    stroke(col);
    ellipse(pos.x, pos.y, rad * 2, rad * 2);
    if(isUnderMouse() || world.selected == this) displayHover();
    if(displayLinks) displayLinks(world.elements);
  }
  public void displayHover()
  {
    noFill();
    stroke(0);
    strokeWeight(1);
    //ellipse(pos.x, pos.y, rad * 2 + 20, rad * 2 + 20);
  }
  public void displayLinks(ArrayList a)
  {
    if(a.indexOf(this) != -1)
    {
      stroke(col, 100);
      for(int i=0; i < childs.size(); i++)
      {
        Element child = (Element)childs.get(i);
        if(a.indexOf(child) != -1)
          line(pos.x, pos.y, child.pos.x, child.pos.y);
      }
    }
  }

  public boolean isUnderMouse()
  {
    if(dist(pos.x, pos.y, mouseX, mouseY) < rad && !mousePressed) return true;
    else return false;
  }

  public float distance(Element e)
  {
    return PVector.dist(pos, e.pos);
  }

  // sorting method
  public int compareTo(Object other)
  {
    int n = 0;
    if(this.childs.size() > ((Element) other).childs.size()) n--;
    else n++;
    if(this.age > ((Element) other).age)
      n -= PApplet.parseInt((this.age - ((Element) other).age) / 50);
    else n += PApplet.parseInt((((Element) other).age - this.age) / 50);

    if(n == 0) return 0;
    else return n/abs(n);
  }
}















class Food extends Element
{
  Food(World world)
  {
    super(world);
    rad = energy * 30 / genes.get("energyStart", 2) + 5; 
    mass = rad * 2;
  }
  Food(World w, PVector p)
  {
    super(w, p);
    rad = energy * 30 / genes.get("energyStart", 2) + 5; 
    mass = rad * 2;
  }
  Food(Food parent)
  {
    super(parent);
  }

  public void update()
  {     
    // life
    energy += world.light * genes.get("feedEffect") / 10;

    // reproduction
    if(energy > 1.5f * genes.get("energyStart") && genes.get("reproductionAbility") > random(2))
    {
      new Food(this);
      energy += genes.get("reproductionEffect");
    }

    /* the life state of close elements affect feeding.
     float closeFoods = 0;
     for(int i=0; i < world.size(); i++)
     {
     Element e = (Element)world.get(i);
     if(e instanceof Food && e.distance(this) <= rad + e.rad + 5)
     closeFoods += (e.energy - e.genes.get("energyStart")) / 20;
     }
     energy += (genes.get("feedEffect") * closeFoods) / frameRate;
     */

    // color
    int test = PApplet.parseInt(energy * 255 / genes.get("energyStart", 2));
    col = color(255 - test,  100 + test, 0);

    // radius
    if(energy < genes.get("energyStart"))
      rad = energy * 30 / genes.get("energyStart", 2) + 5; 

    // do all other things from super class
    super.update();
    cohesion(childs, 5);
  }

  public void display()
  {
    fill(col, 50);
    stroke(col);
    strokeWeight(4);
    super.display();
  }
}


/**
 * FpsCounter class for Processing, 
 * Nicolas Liautaud 2009
 * <br/>
 * Using exemple :
 * 
 * FpsCounter fps = new FpsCounter();
 * void draw(){
 * fps.display();
 * }
 */
 
FpsCounter fps = new FpsCounter();

class FpsCounter
{
  PVector position;
  float size;
  PFont font;
  int digits;
  private boolean change;
  private float fps, time;

  FpsCounter(){
    init(0,15);
  }
  FpsCounter(int x, int y){
    init(x,y);
  }
  private void init(int x, int y){
    change = true;
    position = new PVector(x, y);
    size = 10;
    digits = 1;
    font = createFont("Arial",1);
  }
  public void display(){
    time = millis();
    if(change && time%2 == 0){
      fps = roundTo(frameRate, digits);
      change = false;
    }
    else if(!change && time%2 != 0) change = true;
    display(fps);
  }
  private void display(float fps){
    textFont(font, size);
    fill(0);
    text("FPS " + fps, position.x, position.y);
  }
  public float roundTo(float a, int d){
    return round(a * pow(10,d)) / pow(10,d);
  }
}



class Genome
{
  ArrayList genes;

  Genome()
  {
    genes = new ArrayList();
  }
  
  public int size()
  {
    return genes.size();
  }
  
  public void add(Gene g)
  {
    genes.add(g);
  }
  public float add(String name, float mn, float mx)
  {
    float val = random(mn, mx);
    add(name, mn,  mx, val);
    return val;
  }
  public void add(String name, float mn, float mx, float val)
  {
    Gene g = new Gene(name, mn, mx, val);
    genes.add(g);
  }

  public int getIndex(String name)
  {
    for(int i=0; i < genes.size(); i++)
    {
      ArrayList gene = (ArrayList)genes.get(i);
      if(gene.get(0) == name)
        return i;
    }
    return -1;
  }

  public Gene get(int geneIndex)
  {
    if(geneIndex > -1)
      return (Gene)genes.get(geneIndex);
    else return null;
  }
  
  public Float get(String name)
  {
    return get(name, 3); // by default get the gene value
  }
  public Float get(String name, int valueIndex)
  {
    if(valueIndex > 0)
    {
      for(int i=0; i < genes.size(); i++)
      {
        Gene gene = (Gene)genes.get(i);
        if(gene.name() == name)
          return (Float)gene.get(valueIndex);
      }
    }
    return null;
  }

  public void set(String geneName, float value)
  {
    set(geneName, 3, value); // by default set the gene value
  }
  public void set(String geneName, int index, float value)
  {
    int i = getIndex(geneName);
    Gene gene = (Gene)genes.get(i);
    gene.set(index, value);
    genes.set(i, gene);
  }

  public void print()
  {
    for(int i=0; i < genes.size(); i++)
      println(genes.get(i));
  }
}

class Gene
{
  ArrayList gene;

  Gene(Gene g)
  {
    gene = new ArrayList(4);
    for(int i=0; i < g.gene.size(); i++)
      gene.add(g.gene.get(i));
  }
  Gene(String name, float mn, float mx)
  {
    this(name, mn,  mx, random(mn, mx));
  }
  Gene(String name, float mn, float mx, float val)
  {
    gene = new ArrayList(4);
    gene.add(name);
    gene.add(mn);
    gene.add(mx);
    gene.add(val);
  }

  public Float get()
  {
    return (Float)gene.get(3); // by default get the gene value
  }
  public Float get(int i)
  {
    return (Float)gene.get(i);
  }

  public void set(float value)
  {
    gene.set(3, value); // by default set the gene value
  }
  public void set(int index, float value)
  {
    gene.set(index, value);
  }
  
  public String name()
  {
    return (String)gene.get(0);
  }
}
class InfoBox
{
  World world;
  int boxWidth, boxMargin;
  int titleHeight;
  int barHeight, barSpacing;

  InfoBox(World w)
  {
    world = w;
    boxWidth = 100;
    boxMargin = 2;
    titleHeight = 62;
    barHeight = 12;
    barSpacing = 5;
  }

  // elements infos
  public void display(Element e)
  {
    display(e, PApplet.parseInt(e.pos.x), PApplet.parseInt(e.pos.y), e.rad);
  }
  public void display(Element e, int targetX, int targetY, float biais)
  {
    // data
    PVector test = new PVector(targetX, targetY);
    test.normalize();
    test.mult(e.rad);
    int x = PApplet.parseInt(targetX + test.x);
    int y = PApplet.parseInt(targetY + test.y);
    Genome g = e.genes;
    int bars = g.size() + 1; // genome bars + energy bar
    int boxHeight = titleHeight + bars * (barHeight + barSpacing) - barSpacing + boxMargin;
 
    // box
    fill(255, 200);
    stroke(0);
    strokeWeight(1);
    if(x + boxWidth > width) x = PApplet.parseInt(targetX - boxWidth - biais);
    if(y + boxHeight > height) y = PApplet.parseInt(targetY - boxHeight - biais);
    rect(x, y, boxWidth, boxHeight);
    
    stroke(e.col);
    noFill();
    ellipse(e.pos.x, e.pos.y, e.rad * 2 + 20, e.rad * 2 + 20);
    
    // title
    String title;
    if(e instanceof Boid) title = "Boid, ";
    else if(e instanceof Food) title = "Food, ";
    else title = "";
    if(world.elements.indexOf(e) == -1) title += "#" + (world.deads.indexOf(e)+1);
    else title += "Element #" + world.elements.indexOf(e);

    // attrs
    fill(0);
    text(title + "\nAge :  "+ round(e.age) + "\nMass :  "+ round(e.mass) + "\nChilds : " + e.childs.size(),
    x + boxMargin, y + boxMargin, boxWidth - boxMargin, titleHeight - boxMargin);

    // energy bar
    bar(x + boxMargin, y + titleHeight, boxWidth - 2 * boxMargin, barHeight, 0, g.get("energyStart", 2), e.energy, "energy");
    // genome-based bars
    for(int i=0; i < e.genes.size(); i++)
    {
      Gene gene = (Gene)g.get(i);
      bar(
      x + boxMargin,
      y + (1+i) * (barHeight + barSpacing) + titleHeight,
      boxWidth - 2 * boxMargin,
      barHeight,
      gene.get(1),
      gene.get(2),
      gene.get(3),
      gene.name()
        );
    }
  }

  // world infos in mouse position
  public void display()
  {
    display(mouseX, mouseY);
  }
  // world infos in specified position
  public void display(int targetX, int targetY)
  {
    fill(255);
    stroke(0);
    strokeWeight(1);
    int w = 120;
    int h = 95; 
    float x = targetX + 10;
    float y = targetY + 20;
    if(x + w > width) x =  targetX - w ;
    if(y + h > height) y = targetY - h ;
    rect(x, y, w, h);

    int margin = 3;
    int r = 2;
    fill(0);
    boolean play;
    text("World"+
      "\nSize : "+world.w+"x"+world.h+
      "\nAlive elements : "+world.elements.size()+
      "\nDead elements : "+world.deads.size()+
      "\nFriction : "+world.friction+
      "\nLight : "+world.light,
    x + margin, y + margin, w - margin, h - margin);
  }

  // display infos about dead elements in all world surface
  public void deads()
  {
    Element e, e2;
    int number = 10;
    int margin = 17;
    int boxHeight;
    int col = 0, row = 0, bars = 0;

    // background
    fill(255);
    strokeWeight(margin);
    rect(world.x, world.y, world.w, world.h);
    
    for(int i=0; i < 5; i++)
    {
      int posX = world.w - ((i+2) * boxWidth);

      e = (Element)world.deads.get(i);
      display(e, posX , margin + 60, 0);
      e.pos = new PVector(posX + boxWidth / 2, margin + 30);
      e.display();
      
      e2 = (Element)world.deads.get(i+5);
      display(e2, posX , world.y + world.h - 270 - margin, 0);
      e2.pos = new PVector(posX + boxWidth / 2, world.y + world.h - 300 - margin);
      e2.display();
      
    }
  }

  public void bar(int x, int y, int w, int h, float min, float max, float value, String title)
  {
    // border
    fill(255);
    stroke(0);
    strokeWeight(1);
    rect(x, y, w, h);

    // bar
    float minVal = value - min;
    float maxVal = abs(max-min);
    int barWidth = PApplet.parseInt(minVal * w / maxVal - 1);

    if(barWidth >= 1) // 1 pixel min
    {
      fill(255 - (minVal * 255 / maxVal), minVal * 255 / maxVal, 0);
      noStroke();
      // multi-bar
      if(minVal > maxVal)
      {
        float d = minVal / maxVal;
        float m = minVal % maxVal;
        rect(x+1, y+1, w - 1, h - 1);
        fill(minVal * 255 / maxVal, minVal * 255 / maxVal, d * 20);
        rect(x+1, y+1, m * w / maxVal, h - 1);  
      }  
      else rect(x+1, y+1, barWidth, h - 1);
    }

    // title
    fill(0);
    text(title, x+2, y, w-4, h);
  }
}





/**
 * Zone class for Processing, 
 * Nicolas Liautaud 2009
 * <br/>
 * Used by BoidsFlock class
 */
class World
{
  int x, y, w, h;
  ArrayList elements, deads;
  Element selected;
  float friction;
  float light;
  boolean play;
  InfoBox infos;

  World()
  {
    x = 0;
    y = 0;
    w = width;
    h = height;
    elements = new ArrayList();
    deads = new ArrayList();
    friction = 0.5f;
    light = 1;
    play = true;
    selected = null;
    infos = new InfoBox(this);
  }

  // Add an element to world.
  public void add(Element e)
  {
    elements.add(e); 
  }

  // Add specified number of elements to world.
  public void add(Element e, int n)
  {
    for(int i=0; i < n; i++)
      elements.add(e); 
  }

  // Delete element from world.
  public void die(Element e)
  {
    die(elements.indexOf(e));
  }
  public void die(int i)
  {
    if(i!=-1)
    {
      Element e = (Element)elements.remove(i);
      deads.add(e);
      Collections.sort(deads);
    }
  }
  // Kill all.
  public void kaboom()
  {
    int s = elements.size();
    for(int i=0; i < s; i++) die(0);
  }

  // Return the number of elements in world.
  public int size()
  {
    return elements.size();
  }

  // Return the specified element.
  public Element get(int i)
  {
    return (Element)elements.get(i);
  }

  // Return a random pos vector in world.
  public PVector randomPos()
  {
    return new PVector(random(x, x + w), random(y, y + h)); 
  }

  // Return the element under mouse, or null.
  public Element isHover()
  {
    return isHover(mouseX, mouseY);
  }
  public Element isHover(int x, int y)
  {
    Element hover = null;
    for(int i = 0; i < elements.size(); i++)
    {
      Element e = (Element)get(i);
      if(dist(x, y, e.pos.x, e.pos.y) < e.rad)
      {
        hover = e;
        break;
      }
    }
    return hover;
  }

  // Update all elements in world.
  public void update()
  {
    if(play)
    {
      for(int i = 0; i < elements.size(); i++)
        ((Element)get(i)).update();   
    }
  }

  // Display a border around world.
  public void displayBorder(int a)
  {
    noFill(); 
    stroke(a);
    rect(x, y, w - 1, h - 1); 
  }

  // Display all elements in world.
  public void display()
  {
    for(int i = 0; i < elements.size(); i++)
      ((Element)get(i)).display();

    if(selected != null){
      infos.display(selected);
    }
    else {
      Element e = isHover();
      if(e != null) infos.display(e);
      else if(mousePressed && mouseButton == RIGHT)
        infos.display();
    }
    
    // pause logo
    if(!play)
    {
      fill(50,60);
      stroke(100,60);
      strokeWeight(10);
      rect((width-200)/2, (height-200)/2, 200, 200);
      text("Pause", width/2-15, height/2-5,30,12);
      strokeWeight(40);
      stroke(100,60);
      line((width-200)/2 + 60, (height-200)/2 + 50, (width-200)/2 + 60, (height-200)/2 + 150 );
      line((width-200)/2 + 140, (height-200)/2 + 50, (width-200)/2 + 140, (height-200)/2 + 150 );
    }
  }
}












  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#F0F0F0", "boids6" });
  }
}
