/**
 * Evolution, 
 * Nicolas Liautaud 2009
 */

World world;
Element selected;
int foodNbr, boidNbr;

boolean displayLinks = true;

void setup() 
{ 
  // canvas
  size(700, 700);
  smooth();

  foodNbr = 100;
  boidNbr = 10;

  // initialize fps counter
  fps = new FpsCounter(width - 50, 10);

  // initialize simulation world
  world = new World();

  // create foods
  for(int i=0; i < foodNbr; i++)
    world.add(new Food(world));
  // create boids
  for(int i=0; i < boidNbr; i++)
    world.add(new Boid(world));
}

void draw() 
{
  background(255);

  // display and update world
  world.display();
  world.update();

  // end by replicators invasion (kill all at low framerate)
  if(world.deads.size() > 0 && frameRate < 20)
    world.kaboom();
  // end by no survivors
  if(world.size() < 1)
  {
    world.play = false;  // pause
    world.infos.deads(); // display winners
  }

  fps.display();
} 

// restart, with the 10 bests of the precedent world if there was
void restart()
{
  // if there is deads
  if(world.deads.size() > 0)
  {
    List winners;
    int deadsNbr = world.deads.size();
    // copy the 10 first if there enough, else copy what is it available
    if(deadsNbr >= 9) winners = world.deads.subList(0, 9);
    else winners = world.deads.subList(0, deadsNbr);
    // re-initialize world
    world = new World();
    // add winners with start energy to new world
    for(int i=0; i < winners.size(); i++)
    {
      Element e = (Element)winners.get(i);
      // e.genes.set("energy", e.genes.get("energyStart"));
      world.add(e);
    }
  }
  else world = new World();

  // create foods
  for(int i=0; i < foodNbr; i++)
    world.add(new Food(world));
  // create boids
  for(int i=0; i < boidNbr; i++)
    world.add(new Boid(world));
}

void mousePressed()
{  
  selected = world.isHover(mouseX, mouseY);
  if(selected != null) // an element is cliqued
  {
    // delete it with middle clic
    if(mouseButton == CENTER)
      world.die(selected);
    // select it with left clic
    else if(mouseButton == LEFT)
    {
      // if already selected, unselect-it
      if(world.selected == selected)
        world.selected = null;
      // else select-it
      else world.selected = selected;
    }
  }
  else // nothing is selected
  {
    // add food with left clic
    if(mouseButton == LEFT)
      world.add(new Food(world, new PVector(mouseX, mouseY)));
  }
}
void mouseDragged()
{  
  if(selected != null)
  {
    selected.pos.x = mouseX;
    selected.pos.y = mouseY;
    selected.vel.x = mouseX - pmouseX;
    selected.vel.y = mouseY - pmouseY;
    world.selected = null;
  }
}

void keyPressed()
{
  // reset on SPACE key
  if(key == ' ')
  {
    restart();
    loop();
  }

  // pause on P key
  if(key == 'p' || key == 'P')
  {
    if(world.play) world.play = false;
    else world.play = true;
  }

  // kill all on K key
  if(key == 'k' || key == 'K')
    world.kaboom();

  // display parent/child links on L key
  if(key == 'l' || key == 'L')
  {
    if(displayLinks) displayLinks = false;
    else displayLinks = true;
  }
}



